@@ -1,7 +1,7 @@
-//STARTHEADER
-// $Id: CompositeJetStructure.cc 3071 2013-04-01 12:52:46Z cacciari $
+//FJSTARTHEADER
+// $Id$
 //
-// Copyright (c) 2005-2011, Matteo Cacciari, Gavin P. Salam and Gregory Soyez
+// Copyright (c) 2005-2024, Matteo Cacciari, Gavin P. Salam and Gregory Soyez
 //
 //----------------------------------------------------------------------
 // This file is part of FastJet.
@@ -12,9 +12,11 @@
 //  (at your option) any later version.
 //
 //  The algorithms that underlie FastJet have required considerable
-//  development and are described in hep-ph/0512210. If you use
+//  development. They are described in the original FastJet paper,
+//  hep-ph/0512210 and in the manual, arXiv:1111.6097. If you use
 //  FastJet as part of work towards a scientific publication, please
-//  include a citation to the FastJet paper.
+//  quote the version you use and include a citation to the manual and
+//  optionally also to hep-ph/0512210.
 //
 //  FastJet is distributed in the hope that it will be useful,
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,9 +26,9 @@
 //  You should have received a copy of the GNU General Public License
 //  along with FastJet. If not, see <http://www.gnu.org/licenses/>.
 //----------------------------------------------------------------------
-//ENDHEADER
+//FJENDHEADER
 
-#include <fastjet/CompositeJetStructure.hh>
+#include "fastjet/CompositeJetStructure.hh"
 
 FASTJET_BEGIN_NAMESPACE      // defined in fastjet/internal/base.hh
 
@@ -150,7 +152,7 @@ double CompositeJetStructure::area_error(const PseudoJet & /*reference*/) const{
 
   double a_err=0;
   for (unsigned i = 0; i < _pieces.size(); i++)
-    a_err += _pieces[i].area();
+    a_err += _pieces[i].area_error();
 
   return a_err;
 }
