@@ -1,7 +1,7 @@
-//STARTHEADER
-// $Id: PseudoJet.cc 3083 2013-04-06 12:15:17Z cacciari $
+//FJSTARTHEADER
+// $Id$
 //
-// Copyright (c) 2005-2011, Matteo Cacciari, Gavin P. Salam and Gregory Soyez
+// Copyright (c) 2005-2024, Matteo Cacciari, Gavin P. Salam and Gregory Soyez
 //
 //----------------------------------------------------------------------
 // This file is part of FastJet.
@@ -12,9 +12,11 @@
 //  (at your option) any later version.
 //
 //  The algorithms that underlie FastJet have required considerable
-//  development and are described in hep-ph/0512210. If you use
+//  development. They are described in the original FastJet paper,
+//  hep-ph/0512210 and in the manual, arXiv:1111.6097. If you use
 //  FastJet as part of work towards a scientific publication, please
-//  include a citation to the FastJet paper.
+//  quote the version you use and include a citation to the manual and
+//  optionally also to hep-ph/0512210.
 //
 //  FastJet is distributed in the hope that it will be useful,
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +26,7 @@
 //  You should have received a copy of the GNU General Public License
 //  along with FastJet. If not, see <http://www.gnu.org/licenses/>.
 //----------------------------------------------------------------------
-//ENDHEADER
+//FJENDHEADER
 
 
 #include "fastjet/Error.hh"
@@ -58,10 +60,55 @@ PseudoJet::PseudoJet(const double px_in, const double py_in, const double pz_in,
   this->_finish_init();
 
   // some default values for the history and user indices
+  // note: no reset of shared pointers needed
   _reset_indices();
-
+  
 }
 
+#ifdef FASTJET_HAVE_THREAD_SAFETY
+/// copy-assignmemt
+///
+/// this has to be explicitly specified since atomic does not support it.
+PseudoJet & PseudoJet::operator=(const PseudoJet & other_pj){
+  _structure = other_pj._structure;
+  _user_info = other_pj._user_info;
+
+  _cluster_hist_index = other_pj._cluster_hist_index;
+  _user_index = other_pj._user_index;
+
+  // copy the remaining information through the reset_momentum call
+  // which properly handles the thread safety, notably related
+  // to caching of the phi and rap values
+  reset_momentum(other_pj);
+  
+  return *this;
+}
+#endif // FASTJET_HAVE_THREAD_SAFETY
+
+//std::shared_ptr: //----------------------------------------------------------------------
+//std::shared_ptr: #ifdef FASTJET_HAVE_THREAD_SAFETY
+//std::shared_ptr: PseudoJet::~PseudoJet(){
+//std::shared_ptr:   _release_jet_from_cs();
+//std::shared_ptr: }
+//std::shared_ptr: 
+//std::shared_ptr: // this has to be called everytime one tries to alter the jet
+//std::shared_ptr: // structural info
+//std::shared_ptr: void PseudoJet::_release_jet_from_cs(){
+//std::shared_ptr:   // check if the jet has the structure of type CSstruct in which case
+//std::shared_ptr:   // we have to check if there is a need for self-deletion of the CS
+//std::shared_ptr:   ClusterSequenceStructure * assoc_css = dynamic_cast<ClusterSequenceStructure *>(_structure.get());
+//std::shared_ptr:   if (assoc_css) {
+//std::shared_ptr:     const ClusterSequence * assoc_cs = assoc_css->associated_cluster_sequence();
+//std::shared_ptr:     if (assoc_cs) assoc_cs->release_pseudojet(*this);
+//std::shared_ptr:   }
+//std::shared_ptr: 
+//std::shared_ptr:   // slightly less efficient version
+//std::shared_ptr:   // if ((has_structure_of<ClusterSequence>()) && (has_valid_cluster_sequence())){
+//std::shared_ptr:   //   associated_cs()->release_pseudojet(*this);
+//std::shared_ptr:   // }
+//std::shared_ptr: }
+//std::shared_ptr: 
+//std::shared_ptr: #endif // FASTJET_HAVE_THREAD_SAFETY
 
 //----------------------------------------------------------------------
 /// do standard end of initialisation
@@ -76,8 +123,53 @@ void PseudoJet::_finish_init () {
   // So we initialise it; penalty is about 0.3ns per PseudoJet out of about
   // 10ns total initialisation time (on a intel Core i7 2.7GHz)
   _rap = pseudojet_invalid_rap;
+
+#ifdef FASTJET_HAVE_THREAD_SAFETY
+  _init_status = Init_NotDone;
+#endif
 }
 
+//----------------------------------------------------------------------
+#ifdef FASTJET_HAVE_THREAD_SAFETY
+void PseudoJet::_ensure_valid_rap_phi() const{
+  //TODO: for _init_status we can use memory_order_release when
+  //writing and memory_order_acquire when reading. Check if that has
+  //an impact on timing
+  if (_init_status!=Init_Done){
+    int expected = Init_NotDone;
+    // if it's 0 (not done), set thing to "operation in progress"
+    // (-1) and do the init
+    //
+    // Note:
+    //   - this has to be the strong version so we can make a single
+    //     test
+    //   - we need to make sure that the -1 is loaded properly
+    //     (success), hence a std::memory_order_seq_cst model
+    //   - the failure model can be anything since we're not going
+    //     to use the value anyway
+    if (_init_status.compare_exchange_strong(expected, Init_InProgress,
+                                             std::memory_order_seq_cst,
+                                             std::memory_order_relaxed)){
+      _set_rap_phi();
+      // Now we can safely set the status flag to Init_Done (after all
+      // physics variables are set). The memory ordering guarantees
+      // synchronisation across threads.
+      _init_status.store(Init_Done, memory_order_release); 
+    } else {
+      // wait until done
+      //
+      // Here we use memory_order_acquire to guarantee that the
+      // rap/phi values set in the thread which actually does the
+      // calculation are synchronised.
+      // Reading https://gcc.gnu.org/wiki/Atomic/GCCMM/AtomicSync in
+      // interesting in this context.
+      while (_init_status.load(memory_order_acquire) != Init_Done);
+    }
+    
+  }
+}
+#endif
+
 //----------------------------------------------------------------------
 void PseudoJet::_set_rap_phi() const {
 
@@ -175,6 +267,10 @@ PseudoJet operator- (const PseudoJet & jet1, const PseudoJet & jet2) {
 //----------------------------------------------------------------------
 // return the product, coeff * jet
 PseudoJet operator* (double coeff, const PseudoJet & jet) {
+  // see the comment in operator*= about ensuring valid rap phi
+  // before a multiplication to handle case of multiplication by
+  // zero, while maintaining rapidity and phi
+  jet._ensure_valid_rap_phi(); 
   //return PseudoJet(coeff*jet.four_mom());
   // the following code is hopefully more efficient
   PseudoJet coeff_times_jet(jet);
@@ -196,41 +292,53 @@ PseudoJet operator/ (const PseudoJet & jet, double coeff) {
 
 //----------------------------------------------------------------------
 /// multiply the jet's momentum by the coefficient
-void PseudoJet::operator*=(double coeff) {
+PseudoJet & PseudoJet::operator*=(double coeff) {
+  // operator*= aims to maintain the rapidity and azimuth
+  // for the PseudoJet; if they have already been evaluated
+  // this is fine, but if they haven't and coeff is sufficiently
+  // small as to cause a zero or underflow result, then a subsequent
+  // invocation of rap or phi will lead to a non-sensical result. 
+  // So, here, we preemptively ensure that rapidity and phi
+  // are correctly cached
+  _ensure_valid_rap_phi(); 
   _px *= coeff;
   _py *= coeff;
   _pz *= coeff;
   _E  *= coeff;
   _kt2*= coeff*coeff;
   // phi and rap are unchanged
+  return *this;
 }
 
 //----------------------------------------------------------------------
 /// divide the jet's momentum by the coefficient
-void PseudoJet::operator/=(double coeff) {
+PseudoJet & PseudoJet::operator/=(double coeff) {
   (*this) *= 1.0/coeff;
+  return *this;
 }
 
 
 //----------------------------------------------------------------------
 /// add the other jet's momentum to this jet
-void PseudoJet::operator+=(const PseudoJet & other_jet) {
+PseudoJet & PseudoJet::operator+=(const PseudoJet & other_jet) {
   _px += other_jet._px;
   _py += other_jet._py;
   _pz += other_jet._pz;
   _E  += other_jet._E ;
   _finish_init(); // we need to recalculate phi,rap,kt2
+  return *this;
 }
 
 
 //----------------------------------------------------------------------
 /// subtract the other jet's momentum from this jet
-void PseudoJet::operator-=(const PseudoJet & other_jet) {
+PseudoJet & PseudoJet::operator-=(const PseudoJet & other_jet) {
   _px -= other_jet._px;
   _py -= other_jet._py;
   _pz -= other_jet._pz;
   _E  -= other_jet._E ;
   _finish_init(); // we need to recalculate phi,rap,kt2
+  return *this;
 }
 
 //----------------------------------------------------------------------
@@ -260,8 +368,8 @@ bool operator==(const PseudoJet & jet, const double val) {
 
 
 //----------------------------------------------------------------------
-/// transform this jet (given in lab) into a jet in the rest
-/// frame of prest
+/// transform this jet (given in the rest frame of prest) into a jet
+/// in the lab frame 
 //
 // NB: code adapted from that in herwig f77 (checked how it worked
 // long ago)
@@ -287,8 +395,8 @@ PseudoJet & PseudoJet::boost(const PseudoJet & prest) {
 
 
 //----------------------------------------------------------------------
-/// transform this jet (given in the rest frame of prest) into a jet
-/// in the lab frame;
+/// transform this jet (given in lab) into a jet in the rest
+/// frame of prest  
 //
 // NB: code adapted from that in herwig f77 (checked how it worked
 // long ago)
@@ -327,6 +435,9 @@ void PseudoJet::set_cached_rap_phi(double rap_in, double phi_in) {
   _rap = rap_in; _phi = phi_in;
   if (_phi >= twopi) _phi -= twopi;
   if (_phi < 0)      _phi += twopi;
+#ifdef FASTJET_HAVE_THREAD_SAFETY
+  _init_status = Init_Done;
+#endif
 }
 
 //----------------------------------------------------------------------
@@ -394,11 +505,11 @@ double PseudoJet::delta_phi_to(const PseudoJet & other) const {
 
 string PseudoJet::description() const{
   // the "default" case of a PJ which does not belong to any cluster sequence
-  if (!_structure())
+  if (!_structure)
     return "standard PseudoJet (with no associated clustering information)";
   
   // for all the other cases, the description comes from the structure
-  return _structure()->description();
+  return _structure->description();
 }
 
 
@@ -414,7 +525,7 @@ string PseudoJet::description() const{
 // check whether this PseudoJet has an associated parent
 // ClusterSequence
 bool PseudoJet::has_associated_cluster_sequence() const{
-  return (_structure()) && (_structure->has_associated_cluster_sequence());
+  return (_structure) && (_structure->has_associated_cluster_sequence());
 }
 
 //----------------------------------------------------------------------
@@ -431,7 +542,7 @@ const ClusterSequence* PseudoJet::associated_cluster_sequence() const{
 // check whether this PseudoJet has an associated parent
 // ClusterSequence that is still valid
 bool PseudoJet::has_valid_cluster_sequence() const{
-  return (_structure()) && (_structure->has_valid_cluster_sequence());
+  return (_structure) && (_structure->has_valid_cluster_sequence());
 }
 
 //----------------------------------------------------------------------
@@ -448,13 +559,17 @@ const ClusterSequence * PseudoJet::validated_cs() const {
 //----------------------------------------------------------------------
 // set the associated structure
 void PseudoJet::set_structure_shared_ptr(const SharedPtr<PseudoJetStructureBase> &structure_in){
+//std::shared_ptr: #ifdef FASTJET_HAVE_THREAD_SAFETY
+//std::shared_ptr:   // if the jet currently belongs to a cs, we need to release it before any chenge
+//std::shared_ptr:   _release_jet_from_cs();
+//std::shared_ptr: #endif //FASTJET_HAVE_THREAD_SAFETY
   _structure = structure_in;
 }
 
 //----------------------------------------------------------------------
-// return true if there is some strusture associated with this PseudoJet
+// return true if there is some structure associated with this PseudoJet
 bool PseudoJet::has_structure() const{
-  return _structure();
+  return (bool) _structure; // cast to bool has to be made explicit
 }
 
 //----------------------------------------------------------------------
@@ -463,8 +578,8 @@ bool PseudoJet::has_structure() const{
 //
 // return NULL if there is no associated structure
 const PseudoJetStructureBase* PseudoJet::structure_ptr() const {
-  if (!_structure()) return NULL;
-  return _structure();
+  //if (!_structure) return NULL;
+  return _structure.get();
 }
   
 //----------------------------------------------------------------------
@@ -478,8 +593,8 @@ const PseudoJetStructureBase* PseudoJet::structure_ptr() const {
 // unless you really need a write access to the PseudoJet's
 // underlying structure.
 PseudoJetStructureBase* PseudoJet::structure_non_const_ptr(){
-  if (!_structure()) return NULL;
-  return _structure();
+  //if (!_structure) return NULL;
+  return _structure.get();
 }
   
 //----------------------------------------------------------------------
@@ -488,9 +603,9 @@ PseudoJetStructureBase* PseudoJet::structure_non_const_ptr(){
 //
 // throw an error if there is no associated structure
 const PseudoJetStructureBase* PseudoJet::validated_structure_ptr() const {
-  if (!_structure()) 
+  if (!_structure) 
     throw Error("Trying to access the structure of a PseudoJet which has no associated structure");
-  return _structure();
+  return _structure.get();
 }
   
 //----------------------------------------------------------------------
@@ -558,7 +673,7 @@ bool PseudoJet::is_inside(const PseudoJet &jet) const{
 //----------------------------------------------------------------------
 // returns true if the PseudoJet has constituents
 bool PseudoJet::has_constituents() const{
-  return (_structure()) && (_structure->has_constituents());
+  return (_structure) && (_structure->has_constituents());
 }
 
 //----------------------------------------------------------------------
@@ -571,7 +686,7 @@ vector<PseudoJet> PseudoJet::constituents() const{
 //----------------------------------------------------------------------
 // returns true if the PseudoJet has support for exclusive subjets
 bool PseudoJet::has_exclusive_subjets() const{
-  return (_structure()) && (_structure->has_exclusive_subjets());
+  return (_structure) && (_structure->has_exclusive_subjets());
 }
 
 //----------------------------------------------------------------------
@@ -586,7 +701,7 @@ bool PseudoJet::has_exclusive_subjets() const{
 //
 // an Error is thrown if this PseudoJet has no currently valid
 // associated ClusterSequence
-std::vector<PseudoJet> PseudoJet::exclusive_subjets (const double & dcut) const {
+std::vector<PseudoJet> PseudoJet::exclusive_subjets (const double dcut) const {
   return validated_structure_ptr()->exclusive_subjets(*this, dcut);
 }
 
@@ -597,7 +712,7 @@ std::vector<PseudoJet> PseudoJet::exclusive_subjets (const double & dcut) const
 //
 // an Error is thrown if this PseudoJet has no currently valid
 // associated ClusterSequence
-int PseudoJet::n_exclusive_subjets(const double & dcut) const {
+int PseudoJet::n_exclusive_subjets(const double dcut) const {
   return validated_structure_ptr()->n_exclusive_subjets(*this, dcut);
 }
 
@@ -655,7 +770,7 @@ double PseudoJet::exclusive_subdmerge_max(int nsub) const {
 // By default a single particle or a jet coming from a
 // ClusterSequence have no pieces and this methos will return false.
 bool PseudoJet::has_pieces() const{
-  return ((_structure()) && (_structure->has_pieces(*this)));
+  return ((_structure) && (_structure->has_pieces(*this)));
 }
 
 // retrieve the pieces that make up the jet. 
@@ -751,35 +866,6 @@ void sort_indices(vector<int> & indices,
 }
 
 
-
-//----------------------------------------------------------------------
-/// given a vector of values with a one-to-one correspondence with the
-/// vector of objects, sort objects into an order such that the
-/// associated values would be in increasing order
-template<class T> vector<T>  objects_sorted_by_values(
-                       const vector<T> & objects, 
-		       const vector<double> & values) {
-
-  assert(objects.size() == values.size());
-
-  // get a vector of indices
-  vector<int> indices(values.size());
-  for (size_t i = 0; i < indices.size(); i++) {indices[i] = i;}
-  
-  // sort the indices
-  sort_indices(indices, values);
-  
-  // copy the objects 
-  vector<T> objects_sorted(objects.size());
-  
-  // place the objects in the correct order
-  for (size_t i = 0; i < indices.size(); i++) {
-    objects_sorted[i] = objects[indices[i]];
-  }
-
-  return objects_sorted;
-}
-
 //----------------------------------------------------------------------
 /// return a vector of jets sorted into decreasing kt2
 vector<PseudoJet> sorted_by_pt(const vector<PseudoJet> & jets) {
@@ -846,6 +932,7 @@ PseudoJet join(const PseudoJet & j1){
 // build a "CompositeJet" from two PseudoJet
 PseudoJet join(const PseudoJet & j1, const PseudoJet & j2){
   vector<PseudoJet> pieces;
+  pieces.reserve(2);
   pieces.push_back(j1);
   pieces.push_back(j2);
   return join(pieces);
@@ -854,6 +941,7 @@ PseudoJet join(const PseudoJet & j1, const PseudoJet & j2){
 // build a "CompositeJet" from 3 PseudoJet
 PseudoJet join(const PseudoJet & j1, const PseudoJet & j2, const PseudoJet & j3){
   vector<PseudoJet> pieces;
+  pieces.reserve(3);
   pieces.push_back(j1);
   pieces.push_back(j2);
   pieces.push_back(j3);
@@ -863,6 +951,7 @@ PseudoJet join(const PseudoJet & j1, const PseudoJet & j2, const PseudoJet & j3)
 // build a "CompositeJet" from 4 PseudoJet
 PseudoJet join(const PseudoJet & j1, const PseudoJet & j2, const PseudoJet & j3, const PseudoJet & j4){
   vector<PseudoJet> pieces;
+  pieces.reserve(4);
   pieces.push_back(j1);
   pieces.push_back(j2);
   pieces.push_back(j3);
