@@ -1,7 +1,7 @@
-//STARTHEADER
-// $Id: MassDropTagger.cc 3249 2013-10-28 15:06:25Z salam $
+//FJSTARTHEADER
+// $Id$
 //
-// Copyright (c) 2005-2011, Matteo Cacciari, Gavin P. Salam and Gregory Soyez
+// Copyright (c) 2005-2024, Matteo Cacciari, Gavin P. Salam and Gregory Soyez
 //
 //----------------------------------------------------------------------
 // This file is part of FastJet.
@@ -12,9 +12,11 @@
 //  (at your option) any later version.
 //
 //  The algorithms that underlie FastJet have required considerable
-//  development and are described in hep-ph/0512210. If you use
+//  development. They are described in the original FastJet paper,
+//  hep-ph/0512210 and in the manual, arXiv:1111.6097. If you use
 //  FastJet as part of work towards a scientific publication, please
-//  include a citation to the FastJet paper.
+//  quote the version you use and include a citation to the manual and
+//  optionally also to hep-ph/0512210.
 //
 //  FastJet is distributed in the hope that it will be useful,
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,15 +26,16 @@
 //  You should have received a copy of the GNU General Public License
 //  along with FastJet. If not, see <http://www.gnu.org/licenses/>.
 //----------------------------------------------------------------------
-//ENDHEADER
+//FJENDHEADER
 
-#include <fastjet/tools/MassDropTagger.hh>
-#include <fastjet/ClusterSequence.hh>
+#include "fastjet/tools/MassDropTagger.hh"
+#include "fastjet/ClusterSequence.hh"
 #include <sstream>
 
 FASTJET_BEGIN_NAMESPACE
 
 LimitedWarning MassDropTagger::_warnings_nonca;
+LimitedWarning MassDropTagger::_negative_mass_warning;
 
 using namespace std;
 
@@ -68,6 +71,11 @@ PseudoJet MassDropTagger::result(const PseudoJet & jet) const{
   // appropriate errors will be thrown automatically if this is not
   // the case
   while ((had_parents = j.has_parents(j1,j2))) {
+    if (j.m2() <= 0) {
+      _negative_mass_warning.warn(
+           "MassDropTagger: parent (sub)jet has mass^2<=0; returning null jet");
+      return PseudoJet();
+    }
     // make parent1 the more massive jet
     if (j1.m2() < j2.m2()) std::swap(j1,j2);
 
@@ -88,9 +96,8 @@ PseudoJet MassDropTagger::result(const PseudoJet & jet) const{
   // create the result and its structure
   PseudoJet result_local = j;
   MassDropTaggerStructure * s = new MassDropTaggerStructure(result_local);
-//  s->_original_jet = jet;
-  s->_mu = (j.m2()!=0.0) ? sqrt(j1.m2()/j.m2()) : 0.0;
-  s->_y  = (j.m2()!=0.0) ? j1.kt_distance(j2)/j.m2() : 0.0;
+  s->_mu = j1.m() / j.m();
+  s->_y  = j1.kt_distance(j2)/j.m2();
 
   result_local.set_structure_shared_ptr(SharedPtr<PseudoJetStructureBase>(s));
 
